<?php
/**
 * This file is part of FacturaScripts
 * Copyright (C) 2019-2023 Carlos Garcia Gomez <carlos@facturascripts.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

namespace FacturaScripts\Core\Lib\Email;

use FacturaScripts\Core\Base\ToolBox;
use FacturaScripts\Core\Html;
use FacturaScripts\Core\Model\User;
use FacturaScripts\Core\Tools;
use FacturaScripts\Dinamic\Model\EmailSent;
use FacturaScripts\Dinamic\Model\Empresa;
use PHPMailer\PHPMailer\Exception;
use PHPMailer\PHPMailer\PHPMailer;
use Twig\Error\LoaderError;
use Twig\Error\RuntimeError;
use Twig\Error\SyntaxError;

/**
 * Description of NewMail
 *
 * @author Carlos Garcia Gomez <carlos@facturascripts.com>
 */
class NewMail
{
    const ATTACHMENTS_TMP_PATH = 'MyFiles/Tmp/Email/';
    const DEFAULT_TEMPLATE = 'NewTemplate.html.twig';

    /** @var Empresa */
    public $empresa;

    /** @var string */
    public $fromEmail;

    /** @var string */
    public $fromName;

    /** @var string */
    public $fromNick;

    /** @var BaseBlock[] */
    protected $footerBlocks = [];

    /** @var string */
    protected $html = '';

    /** @var bool */
    protected $lowsecure;

    /** @var PHPMailer */
    protected $mail;

    /** @var BaseBlock[] */
    protected $mainBlocks = [];

    /** @var string */
    public $signature;

    /** @var string */
    public $template;

    /** @var string */
    public $text;

    /** @var string */
    public $title;

    /** @var string */
    public $verificode;

    /** @throws Exception */
    public function __construct()
    {
        $appSettings = $this->toolBox()->appSettings();

        $this->empresa = new Empresa();
        $this->empresa->loadFromCode($appSettings->get('default', 'idempresa'));

        $this->fromEmail = $appSettings->get('email', 'email');
        $this->fromName = $this->empresa->nombrecorto;

        $this->mail = new PHPMailer();
        $this->mail->CharSet = PHPMailer::CHARSET_UTF8;
        $this->mail->Mailer = $appSettings->get('email', 'mailer');

        $this->mail->SMTPSecure = $appSettings->get('email', 'enc', '');
        if ($this->mail->SMTPSecure) {
            $this->mail->SMTPAuth = true;
            $this->mail->AuthType = $appSettings->get('email', 'authtype', '');
        }

        $this->mail->Host = $appSettings->get('email', 'host');
        $this->mail->Port = $appSettings->get('email', 'port');
        $this->mail->Username = $appSettings->get('email', 'user') ?
            $appSettings->get('email', 'user') :
            $appSettings->get('email', 'email');
        $this->mail->Password = $appSettings->get('email', 'password');
        $this->lowsecure = (bool)$appSettings->get('email', 'lowsecure');

        foreach (static::splitEmails($appSettings->get('email', 'emailcc', '')) as $email) {
            $this->addCC($email);
        }

        foreach (static::splitEmails($appSettings->get('email', 'emailbcc', '')) as $email) {
            $this->addBCC($email);
        }

        $this->signature = $appSettings->get('email', 'signature', '');
        $this->template = self::DEFAULT_TEMPLATE;
        $this->verificode = $this->toolBox()->utils()->randomString(20);
    }

    /**
     * @throws Exception
     */
    public function addAddress(string $email, string $name = ''): void
    {
        $this->mail->addAddress($email, $name);
    }

    /**
     * Añade un adjunto al correo.
     *
     * @throws Exception
     */
    public function addAttachment(string $path, string $name): void
    {
        $this->mail->addAttachment($path, $name);
    }

    /**
     * Añade un email con copia oculta.
     *
     * @throws Exception
     */
    public function addBCC(string $email, string $name = ''): void
    {
        $this->mail->addBCC($email, $name);
    }

    /**
     * Añade un email con copia.
     *
     * @throws Exception
     */
    public function addCC(string $email, string $name = ''): void
    {
        $this->mail->addCC($email, $name);
    }

    /**
     * Añade un bloque al pie del correo.
     */
    public function addFooterBlock(BaseBlock $block): void
    {
        $block->setVerificode($this->verificode);
        $this->footerBlocks[] = $block;
    }

    /**
     * Añade un bloque al cuerpo del correo.
     */
    public function addMainBlock(BaseBlock $block): void
    {
        $block->setVerificode($this->verificode);
        $this->mainBlocks[] = $block;
    }

    /**
     * Añade un email de respuesta.
     *
     * @throws Exception
     */
    public function addReplyTo(string $address, string $name = ''): void
    {
        $this->mail->addReplyTo($address, $name);
    }

    /**
     * Verifica si se puede enviar el correo.
     */
    public function canSendMail(): bool
    {
        return !empty($this->fromEmail) && !empty($this->mail->Password) && !empty($this->mail->Host);
    }

    public static function getAttachmentPath(?string $email, string $folder): string
    {
        $path = 'MyFiles/Email/{{email}}/' . $folder . '/';
        return str_replace('{{email}}', $email, $path);
    }

    /**
     * Devuelve los nombres de los archivos adjuntos.
     */
    public function getAttachmentNames(): array
    {
        $names = [];
        foreach ($this->mail->getAttachments() as $attach) {
            $names[] = $attach[1];
        }

        return $names;
    }

    /**
     * Devuelve un array con los emails disponibles para el usuario.
     */
    public function getAvailableMailboxes(): array
    {
        return empty($this->fromEmail) ? [] : [$this->fromEmail];
    }

    /**
     * Devuelve un array con los emails con copia oculta.
     */
    public function getBCCAddresses(): array
    {
        $addresses = [];
        foreach ($this->mail->getBccAddresses() as $addr) {
            $addresses[] = $addr[0];
        }

        return $addresses;
    }

    /**
     * Devuelve un array con los emails con copia.
     */
    public function getCCAddresses(): array
    {
        $addresses = [];
        foreach ($this->mail->getCcAddresses() as $addr) {
            $addresses[] = $addr[0];
        }

        return $addresses;
    }

    /**
     * Devuelve un array con los emails hacia donde va el mensaje.
     */
    public function getToAddresses(): array
    {
        $addresses = [];
        foreach ($this->mail->getToAddresses() as $addr) {
            $addresses[] = $addr[0];
        }

        return $addresses;
    }

    /**
     * Envía el correo.
     *
     * @return bool
     * @throws Exception
     * @throws LoaderError
     * @throws RuntimeError
     * @throws SyntaxError
     */
    public function send(): bool
    {
        if (empty($this->mail->Username) || empty($this->mail->Password)) {
            $this->toolBox()->i18nLog()->warning('email-not-configured');
            return false;
        }

        $this->mail->setFrom($this->fromEmail, $this->fromName);
        $this->mail->Subject = $this->title;

        $this->renderHTML();
        $this->mail->msgHTML($this->html);

        if ('smtp' === $this->mail->Mailer && false === $this->mail->smtpConnect($this->smtpOptions())) {
            $this->toolBox()->i18nLog()->warning('mail-server-error');
            return false;
        }

        if ($this->mail->send()) {
            $this->saveMailSent();
            return true;
        }

        $this->toolBox()->i18nLog()->error('error', ['%error%' => $this->mail->ErrorInfo]);
        return false;
    }

    public function setMailbox(string $emailFrom)
    {
        return $this;
    }

    /**
     * Establece el usuario que manda el email.
     */
    public function setUser(User $user)
    {
        $this->fromNick = $user->nick;

        return $this;
    }

    /**
     * Separa los emails de una cadena en array.
     */
    public static function splitEmails(string $emails): array
    {
        $return = [];
        foreach (explode(',', $emails) as $part) {
            $email = trim($part);
            if (!empty($part)) {
                $return[] = $email;
            }
        }

        return $return;
    }

    /**
     * Pruebe la conexión PHPMailer.
     *
     * @throws Exception
     */
    public function test(): bool
    {
        switch ($this->mail->Mailer) {
            case 'smtp':
                $this->mail->SMTPDebug = 3;
                return $this->mail->smtpConnect($this->smtpOptions());

            default:
                $this->toolBox()->i18nLog()->warning('test-' . $this->mail->Mailer . '-not-implemented');
                return false;
        }
    }

    /**
     * Devuelve los bloques del pie del correo.
     */
    protected function getFooterBlocks(): array
    {
        $signature = $this->toolBox()->utils()->fixHtml($this->signature);
        return array_merge([new TextBlock($signature)], $this->footerBlocks);
    }

    /**
     * Devuelve los bloques del cuerpo del correo.
     */
    protected function getMainBlocks(): array
    {
        return array_merge([new TextBlock($this->text)], $this->mainBlocks);
    }

    /**
     * Renderiza el HTML del correo.
     *
     * @throws SyntaxError
     * @throws RuntimeError
     * @throws LoaderError
     */
    protected function renderHTML(): void
    {
        $params = [
            'empresa' => $this->empresa,
            'footerBlocks' => $this->getFooterBlocks(),
            'mainBlocks' => $this->getMainBlocks(),
            'title' => $this->title
        ];

        $this->html = Html::render('Email/' . $this->template, $params);
    }

    /**
     * Guarda el correo enviado en la base de datos.
     */
    protected function saveMailSent(): void
    {
        // Obtiene todas las direcciones de correo electrónico
        $addresses = array_merge($this->getToAddresses(), $this->getCcAddresses(), $this->getBccAddresses());

        // Generamos un identificador único para el correo electrónico
        $uuid = uniqid();

        // obtenemos los adjuntos
        $attachments = $this->mail->getAttachments();

        // guardar correo electrónico enviado
        foreach (array_unique($addresses) as $address) {
            $emailSent = new EmailSent();
            $emailSent->addressee = $address;
            $emailSent->attachment = count($attachments) > 0;
            $emailSent->body = $this->text;
            $emailSent->email_from = $this->fromEmail;
            $emailSent->html = $this->html;
            $emailSent->nick = $this->fromNick;
            $emailSent->subject = $this->title;
            $emailSent->uuid = $uuid;
            $emailSent->verificode = $this->verificode;
            $emailSent->save();
        }

        // si no hay adjuntos, terminamos
        if (empty($attachments)) {
            return;
        }

        // creamos la carpeta de adjuntos para el email
        $path = FS_FOLDER . '/' . static::getAttachmentPath($this->fromEmail, 'Sent') . $uuid . '/';
        Tools::folderCheckOrCreate($path);

        // movemos los adjuntos a la carpeta temporal a la carpeta de adjuntos del email
        foreach ($attachments as $attach) {
            $tmpPath = FS_FOLDER . '/' . static::ATTACHMENTS_TMP_PATH . $attach[1];
            if (file_exists($tmpPath)) {
                $newPath = $path . $attach[1];
                rename($tmpPath, $newPath);
            }
        }
    }

    /**
     * Devuelve las opciones SMTP.
     */
    protected function smtpOptions(): array
    {
        if ($this->lowsecure) {
            return [
                'ssl' => [
                    'verify_peer' => false,
                    'verify_peer_name' => false,
                    'allow_self_signed' => true
                ]
            ];
        }

        return [];
    }

    protected function toolBox(): ToolBox
    {
        return new ToolBox();
    }
}
